@@ -4,7 +4,7 @@
 
 -- |
 -- Module      : Database.Relational.Query.Projectable
--- Copyright   : 2013 Kei Hibino
+-- Copyright   : 2013-2017 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
@@ -21,7 +21,7 @@ module Database.Relational.Query.Projectable (
   value,
   valueTrue, valueFalse,
   values,
-  unsafeValueNull,
+  nothing, unsafeValueNull,
 
   -- * Placeholders
   PlaceHolders, unsafeAddPlaceHolders, unsafePlaceHolders,
@@ -67,24 +67,24 @@ module Database.Relational.Query.Projectable (
 import Prelude hiding (pi)
 
 import Data.String (IsString)
-import Data.Monoid ((<>), mconcat)
 import Control.Applicative ((<$>))
 
 import Language.SQL.Keyword (Keyword)
 import qualified Language.SQL.Keyword as SQL
 
 import Database.Record
-  (PersistableWidth, PersistableRecordWidth, derivedWidth,
+  (PersistableWidth, persistableWidth, PersistableRecordWidth, derivedWidth,
    HasColumnConstraint, NotNull)
+import Database.Record.Persistable (runPersistableRecordWidth)
 
 import Database.Relational.Query.Internal.SQL (StringSQL, stringSQL, showStringSQL)
-import Database.Relational.Query.Context (Flat, Aggregated, Exists, OverWindow)
+import qualified Database.Relational.Query.Internal.Sub as Internal
 
+import Database.Relational.Query.Context (Flat, Aggregated, Exists, OverWindow)
 import Database.Relational.Query.Pure
   (ShowConstantTermsSQL, showConstantTermsSQL', ProductConstructor (..))
 import Database.Relational.Query.Pi (Pi)
 import qualified Database.Relational.Query.Pi as Pi
-
 import Database.Relational.Query.Projection
   (Projection, ListProjection)
 import qualified Database.Relational.Query.Projection as Projection
@@ -126,9 +126,19 @@ unsafeProjectSql' =  unsafeProjectSqlTerms' . (:[])
 unsafeProjectSql :: SqlProjectable p => String -> p t
 unsafeProjectSql =  unsafeProjectSql' . stringSQL
 
--- | Polymorphic projection of SQL null value.
-unsafeValueNull :: OperatorProjectable p => p (Maybe a)
-unsafeValueNull =  unsafeProjectSql "NULL"
+-- | Polymorphic projection of SQL null value. Semantics of comparing is unsafe.
+nothing :: (OperatorProjectable (Projection c), SqlProjectable (Projection c), PersistableWidth a)
+        => Projection c (Maybe a)
+nothing = proxyWidth persistableWidth
+  where
+    proxyWidth :: SqlProjectable (Projection c) => PersistableRecordWidth a -> Projection c (Maybe a)
+    proxyWidth w = unsafeProjectSqlTerms' $ replicate (runPersistableRecordWidth w) SQL.NULL
+
+{-# DEPRECATED unsafeValueNull "Use `nothing' instead of this." #-}
+-- | Deprecated. Polymorphic projection of SQL null value.
+unsafeValueNull :: (OperatorProjectable (Projection c), SqlProjectable (Projection c), PersistableWidth a)
+                => Projection c (Maybe a)
+unsafeValueNull = nothing
 
 -- | Generate polymorphic projection of SQL constant values from Haskell value.
 value :: (ShowConstantTermsSQL t, OperatorProjectable p) => t -> p t
@@ -361,59 +371,48 @@ showNumMaybe :: (SqlProjectable p, ProjectableShowSql p, Num a, IsString b)
                    => p (Maybe a) -> p (Maybe b)
 showNumMaybe = unsafeCastProjectable
 
-whensClause :: (OperatorProjectable p, ProjectableShowSql p)
-            => String       -- ^ Error tag
-            -> [(p a, p b)] -- ^ Each when clauses
-            -> p b          -- ^ Else result projection
-            -> Keyword      -- ^ Result projection
-whensClause eTag cs0 e = d cs0  where
-  d []       = error $ eTag ++ ": Empty when clauses!"
-  d cs@(_:_) = mconcat [when' p r | (p, r) <- cs] <> else' <> SQL.END
-  when' p r = SQL.WHEN <> unsafeShowSql' p <> SQL.THEN <> unsafeShowSql' r
-  else'     = SQL.ELSE <> unsafeShowSql' e
-
 -- | Search case operator correnponding SQL search /CASE/.
 --   Like, /CASE WHEN p0 THEN a WHEN p1 THEN b ... ELSE c END/
-caseSearch :: (OperatorProjectable p, ProjectableShowSql p)
-           => [(p (Maybe Bool), p a)] -- ^ Each when clauses
-           -> p a                     -- ^ Else result projection
-           -> p a                     -- ^ Result projection
-caseSearch cs e = unsafeProjectSql' $ SQL.CASE <> whensClause "caseSearch" cs e
+caseSearch :: OperatorProjectable (Projection c)
+           => [(Projection c (Maybe Bool), Projection c a)] -- ^ Each when clauses
+           -> Projection c a                                -- ^ Else result projection
+           -> Projection c a                                -- ^ Result projection
+caseSearch = Internal.caseSearch
 
 -- | Same as 'caseSearch', but you can write like <when list> `casesOrElse` <else clause>.
-casesOrElse :: (OperatorProjectable p, ProjectableShowSql p)
-            => [(p (Maybe Bool), p a)] -- ^ Each when clauses
-            -> p a                     -- ^ Else result projection
-            -> p a                     -- ^ Result projection
+casesOrElse :: OperatorProjectable (Projection c)
+            => [(Projection c (Maybe Bool), Projection c a)] -- ^ Each when clauses
+            -> Projection c a                                -- ^ Else result projection
+            -> Projection c a                                -- ^ Result projection
 casesOrElse = caseSearch
 
 -- | Null default version of 'caseSearch'.
-caseSearchMaybe :: (OperatorProjectable p, ProjectableShowSql p)
-                => [(p (Maybe Bool), p (Maybe a))] -- ^ Each when clauses
-                -> p (Maybe a)                     -- ^ Result projection
+caseSearchMaybe :: (OperatorProjectable (Projection c) {- (Projection c) is always ProjectableMaybe -}, PersistableWidth a)
+                => [(Projection c (Maybe Bool), Projection c (Maybe a))] -- ^ Each when clauses
+                -> Projection c (Maybe a)                                -- ^ Result projection
 caseSearchMaybe cs = caseSearch cs unsafeValueNull
 
 -- | Simple case operator correnponding SQL simple /CASE/.
 --   Like, /CASE x WHEN v THEN a WHEN w THEN b ... ELSE c END/
-case' :: (OperatorProjectable p, ProjectableShowSql p)
-      => p a          -- ^ Projection value to match
-      -> [(p a, p b)] -- ^ Each when clauses
-      -> p b          -- ^ Else result projection
-      -> p b          -- ^ Result projection
-case' v cs e = unsafeProjectSql' $ SQL.CASE <> unsafeShowSql' v <> whensClause "case'" cs e
+case' :: OperatorProjectable (Projection c)
+      => Projection c a                     -- ^ Projection value to match
+      -> [(Projection c a, Projection c b)] -- ^ Each when clauses
+      -> Projection c b                     -- ^ Else result projection
+      -> Projection c b                     -- ^ Result projection
+case' = Internal.case'
 
 -- | Uncurry version of 'case'', and you can write like ... `casesOrElse'` <else clause>.
-casesOrElse' :: (OperatorProjectable p, ProjectableShowSql p)
-             => (p a, [(p a, p b)]) -- ^ Projection value to match and each when clauses list
-             -> p b                 -- ^ Else result projection
-             -> p b                 -- ^ Result projection
+casesOrElse' :: OperatorProjectable (Projection c)
+             => (Projection c a, [(Projection c a, Projection c b)]) -- ^ Projection value to match and each when clauses list
+             -> Projection c b                                       -- ^ Else result projection
+             -> Projection c b                                       -- ^ Result projection
 casesOrElse' =  uncurry case'
 
 -- | Null default version of 'case''.
-caseMaybe :: (OperatorProjectable p, ProjectableShowSql p, ProjectableMaybe p)
-          => p a                  -- ^ Projection value to match
-          -> [(p a, p (Maybe b))] -- ^ Each when clauses
-          -> p (Maybe b)          -- ^ Result projection
+caseMaybe :: (OperatorProjectable (Projection c) {- (Projection c) is always ProjectableMaybe -}, PersistableWidth b)
+          => Projection c a                             -- ^ Projection value to match
+          -> [(Projection c a, Projection c (Maybe b))] -- ^ Each when clauses
+          -> Projection c (Maybe b)                     -- ^ Result projection
 caseMaybe v cs = case' v cs unsafeValueNull
 
 -- | Binary operator corresponding SQL /IN/ .
