@@ -211,6 +211,55 @@ _p_j3s :: IO ()
 _p_j3s =  mapM_ print [show j3left, show j3right]
 
 
+-- Index of Nested Projections
+
+nestedPiRec :: Relation () SetA
+nestedPiRec = relation $ do
+  ar <- query . relation $ do
+    a <- query setA
+    return $ value "Hello" >< a
+  return $ ar ! snd'
+
+nestedPiCol :: Relation () String
+nestedPiCol = relation $ do
+  ar <- query . relation $ do
+    a <- query setA
+    return $ a >< value "Hello"
+  return $ ar ! snd'
+
+nestedPi :: Relation () String
+nestedPi = relation $ do
+  ar <- query . relation $ do
+    a <- query setA
+    return $ (value "Hello" >< a) >< value "World"
+  return $ ar ! snd'
+
+nested :: [Test]
+nested =
+  [ eqProp "nested pi record" nestedPiRec
+    "SELECT ALL T1.f1 AS f0, T1.f2 AS f1, T1.f3 AS f2 \
+    \  FROM (SELECT ALL 'Hello' AS f0, \
+    \                   T0.int_a0 AS f1, T0.str_a1 AS f2, T0.str_a2 AS f3 \
+    \              FROM TEST.set_a T0) T1"
+
+  , eqProp "nested pi column" nestedPiCol
+    "SELECT ALL T1.f3 AS f0 \
+    \      FROM (SELECT ALL T0.int_a0 AS f0, T0.str_a1 AS f1, T0.str_a2 AS f2, \
+    \                       'Hello' AS f3 \
+    \                  FROM TEST.set_a T0) T1"
+
+  , eqProp "nested pi both" nestedPi
+    "SELECT ALL T1.f4 AS f0 \
+    \      FROM (SELECT ALL 'Hello' AS f0, \
+    \                       T0.int_a0 AS f1, T0.str_a1 AS f2, T0.str_a2 AS f3, \
+    \                       'World' AS f4 \
+    \                  FROM TEST.set_a T0) T1"
+  ]
+
+_p_nested :: IO ()
+_p_nested =  mapM_ print [show nestedPiRec, show nestedPiCol, show nestedPi]
+
+
 -- Projection Operators
 
 bin53 :: (Projection Flat Int32 -> Projection Flat Int32 -> Projection Flat r) -> Relation () r
@@ -259,6 +308,63 @@ bin =
   , eqProp "div"   (bin53 (./.)) "SELECT ALL (5 / 3) AS f0"
   ]
 
+caseSearchX :: Relation () String
+caseSearchX = relation $ do
+  return $
+    caseSearch
+    [ (value 2 .=. value (1  :: Int32)            , value "foo")
+    , (value 5 .=. value 3 .+. value (2 :: Int32) , value "bar")
+    , (value "a" .=. value "b"                    , value "baz") ]
+    (value "other")
+
+caseX :: Relation () String
+caseX = relation $ do
+  return $
+    case'
+    (value (5 :: Int32))
+    [ (value 1             , value "foo")
+    , (value 3 .+. value 2 , value "bar")
+    , (value 10            , value "baz") ]
+    (value "other")
+
+caseRecordX :: Relation () Int32
+caseRecordX = relation $ do
+  return $
+    case'
+    (value (5 :: Int32))
+    [ (value 1             , (,) |$| value 1 |*| value "foo")
+    , (value 3 .+. value 2 , (,) |$| value 2 |*| value "bar")
+    , (value 10            , (,) |$| value 3 |*| value "baz") ]
+    ((,) |$| value (0 :: Int32) |*| value "other")
+    ! fst'
+    .*.
+    value 10
+
+caseRecordMaybeX :: Relation () (Maybe (Int32, String))
+caseRecordMaybeX = relation $ do
+  return $
+    caseMaybe
+    (value (5 :: Int32))
+    [ (value (1 :: Int32)  , just $ (,) |$| value (1 :: Int32) |*| value "foo")
+    , (value 3 .+. value 2 , just $ (,) |$| value 2            |*| value "bar") ]
+
+cases :: [Test]
+cases =
+  [ eqProp "caseSearch" caseSearchX
+    "SELECT ALL CASE WHEN (2 = 1) THEN 'foo' WHEN (5 = (3 + 2)) THEN 'bar' WHEN ('a' = 'b') THEN 'baz' ELSE 'other' END AS f0"
+  , eqProp "case"       caseX
+    "SELECT ALL CASE 5 WHEN 1 THEN 'foo' WHEN (3 + 2) THEN 'bar' WHEN 10 THEN 'baz' ELSE 'other' END AS f0"
+  , eqProp "caseRecord" caseRecordX
+    "SELECT ALL (CASE 5 WHEN 1 THEN 1 WHEN (3 + 2) THEN 2 WHEN 10 THEN 3 ELSE 0 END * 10) AS f0"
+  , eqProp "caseRecordMaybe" caseRecordMaybeX
+    "SELECT ALL CASE 5 WHEN 1 THEN 1 WHEN (3 + 2) THEN 2 ELSE NULL END AS f0, \
+              \ CASE 5 WHEN 1 THEN 'foo' WHEN (3 + 2) THEN 'bar' ELSE NULL END AS f1"
+  ]
+
+_p_cases :: IO ()
+_p_cases =
+  mapM_ print [show caseSearchX, show caseX]
+
 nothingX :: Relation () (SetA, Maybe SetB)
 nothingX =  relation $ do
   a <- query setA
@@ -581,7 +687,7 @@ correlated =
 
 tests :: [Test]
 tests =
-  concat [ tables, monadic, directJoins, join3s, bin, uni
+  concat [ tables, monadic, directJoins, join3s, nested, bin, cases, uni
          , groups, orders, partitions, exps, effs, correlated]
 
 main :: IO ()
